#!/usr/bin/env python
"""
Process NICI output and get a file with samples and compound concentrations

Recursively enter into directory and subdirectories looking for a 
specific filename.
Proces the file looking for the sample name and the concentrations of the compounds.
Write the output to CSV file and a Spreadsheet.

Licensed under the GPL-v3. See http://www.gnu.org/licenses/gpl-3.0.html
"""

__author__ = "Xavier Fernandez i Marin"
__version__ = "1.3"
__contact__ = "http://xavier-fim.net"
__license__ = "GPLv3"


# Get the name of the file that we are looking for in the command line
# Otherwise we can't continue
import sys
if (len(sys.argv) > 1):
    target = sys.argv[1]
    print 'Processing files with name ' + target
else:
    print 'Error: a name for the files being processed is required'
    exit(0)
"We are here"
# Load libraries to enter recursively into directories
import fnmatch
import os

# Load libraries for regular expressions
import re


# Output file for the CSV structured file
o = open("out.csv", 'w')
o.write('sample;compound;concentration\n')

# Output file for the spreadsheet
import xlwt
sf = xlwt.Workbook()
sb = sf.add_sheet("mostres")
#style = xlwt.easyxf("", "#.00")


# Counter for files being processed
nmostra = 0

# Loop over the structure tree to process files
matches = []
for root, dirnames, filenames in os.walk('./'):
    for filename in fnmatch.filter(filenames, target):
        matches.append(os.path.join(root, filename))

        # Open the file that is relevant to analyze
        f = open(os.path.join(root, filename), 'r')  # root is name of sample

        # root is the path of the sample according to the file
        # but not the sample name
        sample = os.path.basename(root)
        print 'Path ' + sample

        # Initialize the line counter and the number of compounds in the file
        # being processed
        l = 0
        ncompost = 0

        # Initialize the "do_process" counter
        do_process = False

        # Loop over lines
        for line in f.readlines():
            l = l+1

            # The name of the sample is in the line that starts with 'Sample'
            # If it is empty, raise an error
            samp = re.search(r'Sample +\: (.+)', line)
            if (samp):
                name_sample = samp.group(1).rstrip()
                if (name_sample == ""):
                    print 'Error: Unable to read the name of the sample based on the line "Sample" at file ' + sample
                    exit(0)
                else:
                    print ' Processing sample ' + name_sample

                    # Write the column header of the sample
                    sb.write(ncompost, nmostra+1, name_sample)

            # Look for the expression "Target Compounds" and process the
            # following lines until the "--------" appears
            # First look for the --- and only process if do_process has been
            # started for the "Target Compounds"
            separator_end = re.search(r'----', line)
            if (separator_end):
                do_process = False

            if (do_process):       # If inside the relevant "Target compounds"
                ncompost = ncompost + 1
                words = line.split()
                w = 0
                for word in words:
                    w = w + 1

                    # w=2 is the name of the compound
                    # w=6 is its concentration
                    # TODO, use regular expressions
                    if (w==2) | (w==6):
                        if (nmostra== 0) & (w==2):
                            sb.write(ncompost , nmostra, word)

                        # Transform 'Below' into -1
                        if (word=="Below"):
                            word = -1.0

                        # Transform 'N.D.' into -2
                        if (word=="d"):
                            word = -2.0


                        # Write the output to the CSV and spreadsheet files
                        # First the name of the compound, and then its
                        # concentration
                        if (w==2):
                            o.write(name_sample + ';' + word + ';')
                        if (w==6):
                            o.write(str(word) + '\n')
                            sb.write(ncompost, nmostra+1, float(word))

            # Finally, start the do_process for the following line if it
            # starts with "Target Compounds"
            targcomp = re.search(r'Target Compounds', line)
            if (targcomp):
                do_process = True

        nmostra = nmostra + 1
        f.close()


# Close the output file
o.close()
sf.save("compounds.xls")

